/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.jenkins.results.parser.test.clazz.group;

import com.liferay.jenkins.results.parser.JenkinsResultsParserUtil;
import com.liferay.jenkins.results.parser.PortalGitWorkingDirectory;

import java.io.File;
import java.io.IOException;

import java.util.List;

/**
 * @author Leslie Wong
 */
public class SemVerBaselineBatchTestClassGroup extends BatchTestClassGroup {

	public static class SemVerBaselineBatchTestClass extends BaseTestClass {

		protected static SemVerBaselineBatchTestClass getInstance(
			String batchName, File moduleBaseDir) {

			return new SemVerBaselineBatchTestClass(batchName, moduleBaseDir);
		}

		protected SemVerBaselineBatchTestClass(
			String batchName, File moduleBaseDir) {

			super(moduleBaseDir);

			addTestMethod(batchName);
		}

	}

	protected SemVerBaselineBatchTestClassGroup(
		String batchName, PortalGitWorkingDirectory portalGitWorkingDirectory,
		String testSuiteName) {

		super(batchName, portalGitWorkingDirectory, testSuiteName);

		try {
			excludesPathMatchers.addAll(
				getPathMatchers(
					JenkinsResultsParserUtil.combine(
						"modules.excludes[", batchName, "]"),
					portalGitWorkingDirectory.getWorkingDirectory()));

			includesPathMatchers.addAll(
				getPathMatchers(
					JenkinsResultsParserUtil.combine(
						"modules.includes[", batchName, "]"),
					portalGitWorkingDirectory.getWorkingDirectory()));

			setTestClasses();

			setAxisTestClassGroups();
		}
		catch (IOException ioe) {
			throw new RuntimeException(ioe);
		}
	}

	protected void setTestClasses() throws IOException {
		PortalGitWorkingDirectory portalGitWorkingDirectory =
			getPortalGitWorkingDirectory();

		List<File> moduleDirsList =
			portalGitWorkingDirectory.getModifiedModuleDirsList(
				excludesPathMatchers, includesPathMatchers);

		if (!testRelevantChanges) {
			moduleDirsList = portalGitWorkingDirectory.getModuleDirsList(
				excludesPathMatchers, includesPathMatchers);

			File portalModulesBaseDir = new File(
				portalGitWorkingDirectory.getWorkingDirectory(), "modules");

			List<File> semVerMarkerFiles = JenkinsResultsParserUtil.findFiles(
				portalModulesBaseDir, "\\.lfrbuild-semantic-versioning");

			for (File semVerMarkerFile : semVerMarkerFiles) {
				moduleDirsList.add(semVerMarkerFile.getParentFile());
			}
		}

		for (File moduleDir : moduleDirsList) {
			testClasses.add(
				SemVerBaselineBatchTestClass.getInstance(batchName, moduleDir));
		}
	}

}